@@ -25,9 +25,7 @@
 #include <ostree-core.h>
 #include <ostree-repo.h>
 #include <ostree-mutable-tree.h>
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
 #include <ostree-remote.h>
-#endif
 #include <ostree-repo-file.h>
 #include <ostree-sysroot.h>
 #include <ostree-sysroot-upgrader.h>
@@ -35,15 +33,11 @@
 #include <ostree-bootconfig-parser.h>
 #include <ostree-diff.h>
 #include <ostree-gpg-verify-result.h>
-
-#ifdef OSTREE_ENABLE_EXPERIMENTAL_API
 #include <ostree-ref.h>
 #include <ostree-repo-finder.h>
 #include <ostree-repo-finder-avahi.h>
 #include <ostree-repo-finder-config.h>
 #include <ostree-repo-finder-mount.h>
 #include <ostree-repo-finder-override.h>
-#endif /* OSTREE_ENABLE_EXPERIMENTAL_API */
-
 #include <ostree-autocleanups.h>
 #include <ostree-version.h>
