@@ -96,7 +96,9 @@ deployment_print_status (OstreeSysroot    *sysroot,
   GKeyFile *origin = ostree_deployment_get_origin (deployment);
 
   const char *deployment_status = "";
-  if (is_pending)
+  if (ostree_deployment_is_staged (deployment))
+    deployment_status = " (staged)";
+  else if (is_pending)
     deployment_status = " (pending)";
   else if (is_rollback)
     deployment_status = " (rollback)";
